package collections

import (
	"reflect"
	"strconv"
	"time"

	"git.perx.ru/perxis/perxis-go/pkg/optional"
	"git.perx.ru/perxis/perxis-go/pkg/schema"
	jsoniter "github.com/json-iterator/go"
)

// UnmarshalJSON implements json.Unmarshaler interface
func (c *Collection) UnmarshalJSON(b []byte) error {
	type collection Collection
	var cc, zero collection

	// Пытаемся распарсить как коллекцию
	if err := jsoniter.Unmarshal(b, &cc); err != nil {
		return err
	}

	// Если это не пустая коллекция, то просто присваиваем
	if !reflect.DeepEqual(cc, zero) {
		*c = Collection(cc)
		return nil
	}

	// Пытаемся распарсить как схему
	var s schema.Schema
	if err := jsoniter.Unmarshal(b, &s); err != nil {
		return err
	}
	*c = *FromSchema(&s)
	return nil
}

// FromSchema создает новую коллекцию из схемы
func FromSchema(sch *schema.Schema) *Collection {
	if sch == nil {
		return nil
	}

	coll := &Collection{Schema: sch}

	coll.ID = sch.Metadata["collection_id"]
	coll.Name = sch.Metadata["collection_name"]

	if single, ok := sch.Metadata["collection_single"]; ok && single == "true" {
		coll.Single = optional.True
	}
	if system, ok := sch.Metadata["collection_system"]; ok && system == "true" {
		coll.System = optional.True
	}
	if nodata, ok := sch.Metadata["collection_nodata"]; ok && nodata == "true" {
		coll.NoData = optional.True
	}
	if hidden, ok := sch.Metadata["collection_hidden"]; ok && hidden == "true" {
		coll.Hidden = true
	}
	if disablePublishing, ok := sch.Metadata["collection_no_publish"]; ok && disablePublishing == "true" {
		coll.NoPublish = true
	}
	if noArchive, ok := sch.Metadata["collection_no_archive"]; ok && noArchive == "true" {
		coll.NoArchive = true
	}

	if noRevisions, ok := sch.Metadata["collection_no_revisions"]; ok && noRevisions == "true" {
		coll.NoRevisions = true
	}
	if mr, ok := sch.Metadata["collection_max_revisions"]; ok {
		if maxRevisions, err := strconv.ParseUint(mr, 10, 32); err == nil {
			coll.MaxRevisions = uint32(maxRevisions)
		}
	}
	if ttl, ok := sch.Metadata["collection_revisions_ttl"]; ok {
		if revisionTTL, err := time.ParseDuration(ttl); err == nil {
			coll.RevisionTTL = revisionTTL
		}
	}

	if _, ok := sch.Metadata["collection_view_id"]; ok {
		coll.View = &View{
			SpaceID:      sch.Metadata["collection_view_space"],
			EnvID:        sch.Metadata["collection_view_env"],
			CollectionID: sch.Metadata["collection_view_id"],
			Filter:       sch.Metadata["collection_view_filter"],
		}
	}

	return coll
}
