package middleware

import (
	"context"

	pkgId "git.perx.ru/perxis/perxis-go/id"
	"git.perx.ru/perxis/perxis-go/pkg/collections"
	"git.perx.ru/perxis/perxis-go/pkg/schema"
	logzap "git.perx.ru/perxis/perxis-go/zap"
	"go.uber.org/zap"
)

type loggingMiddleware struct {
	logger *zap.Logger
	next   collections.Collections
}

func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next collections.Collections) collections.Collections {
		return &loggingMiddleware{
			next:   next,
			logger: logger.With(logzap.Component("Collections")),
		}
	}
}

func (m *loggingMiddleware) Create(ctx context.Context, coll *collections.Collection) (created *collections.Collection, err error) {
	var spaceID string
	if coll != nil {
		spaceID = coll.SpaceID
	}
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceID)),
		logzap.Event(collections.EventCreate),
	)

	created, err = m.next.Create(ctx, coll)
	if err != nil {
		logger.Error("Failed to create", zap.Error(err), logzap.Object(coll), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Collection created", logzap.Object(created), logzap.Channels(logzap.Userlog))
	return created, err
}

func (m *loggingMiddleware) Delete(ctx context.Context, spaceId string, envId string, collectionId string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceId)),
		logzap.Event(collections.EventDelete),
		logzap.Object(pkgId.NewCollectionId(spaceId, envId, collectionId)),
	)

	err = m.next.Delete(ctx, spaceId, envId, collectionId)
	if err != nil {
		logger.Error("Failed to delete", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Collection deleted", logzap.Channels(logzap.Userlog))
	return err
}

func (m *loggingMiddleware) Get(ctx context.Context, spaceId string, envId string, id string, options ...*collections.GetOptions) (coll *collections.Collection, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceId)),
	)

	coll, err = m.next.Get(ctx, spaceId, envId, id, options...)
	if err != nil {
		logger.Error("Failed to get", zap.Error(err))
		return
	}

	return coll, err
}

func (m *loggingMiddleware) List(ctx context.Context, spaceId string, envId string, filter *collections.Filter) (colls []*collections.Collection, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceId)),
	)

	colls, err = m.next.List(ctx, spaceId, envId, filter)
	if err != nil {
		logger.Error("Failed to list", zap.Error(err))
		return
	}

	return colls, err
}

func (m *loggingMiddleware) SetSchema(ctx context.Context, spaceId string, envId string, id string, schema *schema.Schema) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceId)),
		logzap.Event(collections.EventSetSchema),
		logzap.Object(pkgId.NewCollectionId(spaceId, envId, id)),
	)

	err = m.next.SetSchema(ctx, spaceId, envId, id, schema)
	if err != nil {
		logger.Error("Failed to set schema", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Set collection's schema", logzap.Channels(logzap.Userlog))
	return err
}

func (m *loggingMiddleware) SetState(ctx context.Context, spaceId string, envId string, id string, state *collections.StateInfo) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceId)),
	)

	err = m.next.SetState(ctx, spaceId, envId, id, state)
	if err != nil {
		logger.Error("Failed to set state", zap.Error(err))
		return
	}

	logger.Info("Set collection's state", logzap.Channels(logzap.Userlog))
	return err
}

func (m *loggingMiddleware) Update(ctx context.Context, coll *collections.Collection) (err error) {
	var spaceID string
	if coll != nil {
		spaceID = coll.SpaceID
	}
	logger := m.logger.With(
		logzap.Caller(ctx, logzap.WithSpace(spaceID)),
		logzap.Event(collections.EventUpdate),
		logzap.Object(coll),
	)

	err = m.next.Update(ctx, coll)
	if err != nil {
		logger.Error("Failed to update", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Collection updated", logzap.Channels(logzap.Userlog))
	return err
}
