package yaml

import (
	"io/fs"

	"gopkg.in/yaml.v3"
)

type TagProcessor interface {
	yaml.Unmarshaler
	FS() fs.FS
}

// tagProcessor обёртка для декодирования целевого значения.
//
// Перед декодированием значения обрабатываются все теги узла.
type tagProcessor struct {
	fsys   fs.FS
	target any
}

// WithTagProcessor возвращает функцию, которая оборачивает декодируемые значения для поддержки обработки тегов YAML.
//
// Для путей к файлам, используемых в качестве значений тегов, пути должны быть указаны относительно переданной файловой системы.
func WithTagProcessor(fsys fs.FS) func(any) *tagProcessor {
	return func(v any) *tagProcessor {
		return &tagProcessor{fsys: fsys, target: v}
	}
}

func (tp *tagProcessor) FS() fs.FS {
	return tp.fsys
}

func (tp *tagProcessor) UnmarshalYAML(value *yaml.Node) error {
	resolved, err := resolveTags(tp, value)
	if err != nil {
		return err
	}
	return resolved.Decode(tp.target)
}

// resolveTags обрабатывает все теги YAML для переданного узла и возвращает исправленный узел.
// Если узел представляет собой последовательность или словарь, то обрабатываются теги для его дочерних элементов.
func resolveTags(tp TagProcessor, node *yaml.Node) (*yaml.Node, error) {
	switch node.Kind {
	case yaml.SequenceNode, yaml.MappingNode:
		for i := range node.Content {
			var err error
			node.Content[i], err = resolveTags(tp, node.Content[i])
			if err != nil {
				return nil, err
			}
		}
	default:
		if resolver, ok := tagResolvers[node.Tag]; ok {
			return resolver.Resolve(tp, node)
		}
	}
	return node, nil
}
